﻿package 
{
	import flash.display.*;
	import flash.events.*;
	import flash.geom.*;
	import flash.media.*;
	import ui.*;
	import org.mangui.hls.utils.Log;
	import org.mangui.hls.utils.ScaleVideo;
	import org.mangui.hls.model.AudioTrack;
	import org.mangui.hls.HLSSettings;
	import org.mangui.hls.event.HLSError;
	import org.mangui.hls.event.HLSEvent;
	import org.mangui.hls.HLS;
	import org.mangui.hls.model.Level;

	public class Main extends MovieClip
	{
		private var _hls:HLS;
		private var _sheet:Sprite;
		private var _stageVideo:StageVideo = null;
		private var _video:Video = null;
		private var _videoWidth:int = 0;
		private var _videoHeight:int = 0;
		private var _media_position:Number;
		private var _duration:Number;
		private var _autoLoad:Boolean = false;
		private var W:int;
		private var H:int;
		private var fullscreen:Boolean = true;
		private var autoplay:Boolean = true;
		private var _nav_bar;
		private var playing:Boolean = true;
		private var _volcache:Number = 0;
		private var st:SoundTransform = new SoundTransform  ;
		private var fs:Boolean = false;

		public function Main()
		{
			_init_ui(stage.stageWidth,stage.stageHeight);
			_init_events();
		}
		private function _init_events()
		{
			stage.addEventListener(StageVideoAvailabilityEvent.STAGE_VIDEO_AVAILABILITY, _onStageVideoState);
			nav.playButton.addEventListener(MouseEvent.CLICK, playOrpauseEvent);
			nav.pauseButton.addEventListener(MouseEvent.CLICK, playOrpauseEvent);
			nav.mute.addEventListener(MouseEvent.CLICK, _mute);
			nav.playingBar.addEventListener(MouseEvent.CLICK, progressBarEvent);
			nav.progressBar.addEventListener(MouseEvent.CLICK, progressBarEvent);
			nav.progressBar.addEventListener(MouseEvent.MOUSE_OVER, seeker_time_show);
			nav.playingBar.addEventListener(MouseEvent.MOUSE_OVER, seeker_time_show);
			nav.progressBar.addEventListener(MouseEvent.MOUSE_MOVE, seeker_time_show);
			nav.playingBar.addEventListener(MouseEvent.MOUSE_MOVE, seeker_time_show);
			nav.progressBar.addEventListener(MouseEvent.MOUSE_OUT, seeker_time_hide);
			nav.playingBar.addEventListener(MouseEvent.MOUSE_OUT, seeker_time_hide);
			nav.volumeBar.addEventListener(MouseEvent.CLICK, volume_bar_click);
			nav.volumn_btn.addEventListener(MouseEvent.MOUSE_DOWN, volumn_btn_mouse_down);
			stage.addEventListener(MouseEvent.MOUSE_UP, volumn_btn_mouse_up);
			nav.fullscreen.addEventListener(MouseEvent.CLICK, fullscreenEvent);
			stage.addEventListener(Event.RESIZE, resizeEvent);
		}
		private function _onStageVideoState(event : StageVideoAvailabilityEvent):void
		{
			var available : Boolean = (event.availability == StageVideoAvailability.AVAILABLE);
			_hls = new HLS();
			HLSSettings.maxBufferLength = 10;//最大缓存长度
			HLSSettings.seekMode = "KEYFRAME";//查找模式
			_hls.stage = stage;
			_hls.addEventListener(HLSEvent.MANIFEST_LOADED, _manifestHandler);
			_hls.addEventListener(HLSEvent.MEDIA_TIME, _mediaTimeHandler);//当前播放回调
			_hls.addEventListener(HLSEvent.FRAGMENT_PLAYING, _fragmentPlayingHandler);//片段播放回调
			_hls.addEventListener(HLSEvent.PLAYBACK_STATE, _stateHandler);//当前状态回调
			if (available && stage.stageVideos.length > 0)
			{
				_stageVideo = stage.stageVideos[0];
				_stageVideo.attachNetStream(_hls.stream);
			}
			else
			{
				_video = new Video();
				addChild(_video);
				swapChildren(nav,_video);
				_video.smoothing = true;
				_video.attachNetStream(_hls.stream);
			}
			stage.removeEventListener(StageVideoAvailabilityEvent.STAGE_VIDEO_AVAILABILITY, _onStageVideoState);
			var autoLoadUrl:String = root.loaderInfo.parameters.url as String;
			if (autoLoadUrl != null)
			{
				_autoLoad = true;
				_load(autoLoadUrl);
			}
			_setVolume(40);
		}
		private function _load(url : String):void
		{
			_hls.load(url);
		}
		private function _manifestHandler(event : HLSEvent):void
		{
			_duration = event.levels[_hls.startlevel].duration;
			if (_autoLoad)
			{
				_play(-1);
			}
		}
		private function _play(position : Number = -1):void
		{
			_hls.stream.play(null, position);
		}
		private function _pause():void
		{
			_hls.stream.pause();
		}
		protected function _resume():void//回放
		{
			_hls.stream.resume();
		}
		private function _seek(position : Number):void
		{
			_hls.stream.seek(position);
		}
		private function _stop():void
		{
			_hls.stream.close();
		}
		public function _mute(e:MouseEvent):Number
		{
			if (_volcache)
			{
				st.volume = _volcache;
				_hls.stream.soundTransform = st;
				_volcache = 0;
			}
			else
			{
				_volcache = st.volume;
				st.volume = 0;
				_hls.stream.soundTransform = st;
			}
			nav.mute.gotoAndStop((st.volume> 0) ? 1 : 2);
			return st.volume;
		}
		private function _setVolume(percent : Number):void
		{
			st.volume = percent / 100;
			_hls.stream.soundTransform = st;
		}
		private function _getVolume():Number
		{
			return _hls.stream.soundTransform.volume;
		}
		private function progressBarEvent(e:MouseEvent):void
		{
			var point:Number = stage.mouseX;
			var seekpoint:Number = (point / nav.bar.width) * _duration;
			_seek(seekpoint);
		}
		private function _stateHandler(event : HLSEvent):void
		{
			if (event.state == 'PLAYING_BUFFERING')
			{
				buffering.visible = true;
			}
			else
			{
				buffering.visible = false;
			}
		}
		private function _fragmentPlayingHandler(event : HLSEvent):void
		{
			_video.width = _videoWidth = event.playMetrics.video_width;
			_video.height = _videoHeight = event.playMetrics.video_height;
			_video.x=(stage.stageWidth-_video.width)*0.5;
			_video.y=(stage.stageHeight-_video.height)*0.5;
		}
		private function playOrpauseEvent(event:MouseEvent):void
		{
			if (playing)
			{
				_pause();
				playing = false;
			}
			else
			{
				_resume();
				playing = true;
			}
			nav.playButton.visible = ! playing;
			nav.pauseButton.visible = playing;
		}
		private function volumeEvent(event:MouseEvent):void
		{
			_setVolume(0);
		}
		private function _mediaTimeHandler(event : HLSEvent):void
		{
			_duration = event.mediatime.duration;
			_media_position = event.mediatime.position;
			nav.progress_line.x = _media_position / _duration * W;
			nav.progressBar.width = (_media_position + event.mediatime.buffer) / _duration * W;
			nav.notify.text = formatTime(_media_position) + " / " + formatTime(_duration);
		}
		private function seeker_time_show(e:MouseEvent):void
		{
			var target_x = stage.mouseX;
			nav.hover_line.x = target_x;
			if (10<target_x&&target_x<W-10)
			{
				nav.hover_line.visible = true;
				nav.seeker_time.visible = true;
				nav.seeker_time.x = target_x;
				nav.seeker_time.mouseon_time.text = formatTime(stage.mouseX / W * _duration);
			}
		}
		private function seeker_time_hide(e:MouseEvent):void
		{
			nav.hover_line.visible = false;
			nav.seeker_time.visible = false;
		}
		private function volume_bar_click(event:MouseEvent):void
		{
			var volume = event.localX / 140;
			nav.mute.gotoAndStop((volume> 0) ? 1 : 2);
			st.volume = volume;
			_hls.stream.soundTransform = st;
			nav.volumn_btn.x = nav.volumeBar.x + event.localX;
			event.stopPropagation();
		}
		private function volumn_btn_mouse_down(e:Event)
		{
			var rect:Rectangle = new Rectangle(nav.volumeBar.x,nav.volumn_btn.y);
			rect.width = nav.volumeBar.width - nav.volumn_btn.width;
			rect.height = 0;
			e.target.startDrag(false,rect);
			nav.volumn_btn.addEventListener(MouseEvent.MOUSE_MOVE, volumn_btn_mouse_move);
			e.stopPropagation();
		}
		private function volumn_btn_mouse_move(event:MouseEvent):void
		{
			var volume=(nav.volumn_btn.x-nav.volumeBar.x)/(140-nav.volumn_btn.width);
			nav.mute.gotoAndStop((volume> 0) ? 1 : 2);
			st.volume = volume;
			_hls.stream.soundTransform = st;
			event.stopPropagation();
		}
		private function volumn_btn_mouse_up(event:MouseEvent):void
		{
			nav.volumn_btn.stopDrag();
			nav.volumn_btn.removeEventListener(MouseEvent.MOUSE_MOVE, volumn_btn_mouse_move);
			event.stopPropagation();
		}
		private function fullscreenEvent(e:Event):void
		{
			if (stage.displayState == StageDisplayState.FULL_SCREEN)
			{
				fs = false;
				stage.displayState = StageDisplayState.NORMAL;
			}
			else
			{
				fs = true;
				stage.displayState = StageDisplayState.FULL_SCREEN;
			}
		}
		private function resizeEvent(e:Event):void
		{
			_init_ui(stage.stageWidth, stage.stageHeight);
			_video.x=(stage.stageWidth-_video.width)*0.5;
			_video.y=(stage.stageHeight-_video.height)*0.5;
		}
		private function _init_ui(W,H)
		{
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			setChildIndex(nav,2);
			new Button(nav.playButton);
			new Button(nav.pauseButton);
			new Button(nav.fullscreen);
			new Button(nav.volumn_btn);
			new Button(nav.mute);
			this.W = W;
			this.H = H;
			background.x = overlay.x = 0;
			background.y = overlay.y = 0;
			background.width = overlay.width = W;
			background.height = overlay.height = H;
			overlay.alpha = 0;
			buffering.x = (W - buffering.width) * .5;
			buffering.y = (H - buffering.height) * .5;
			//buffering.visible=false;
			nav.bar.width = W;
			nav.pauseButton.y = nav.playButton.y;
			nav.progressBar.x = nav.playingBar.x = 0;
			nav.progressBar.y = nav.playingBar.y = nav.bar.y - 10;
			nav.seeker_time.visible = false;
			nav.seeker_time.y = nav.progressBar.y - 20;
			nav.seeker_time.x = 0;
			if (fullscreen)
			{
				nav.fullscreen.x = nav.bar.width - nav.fullscreen.width - 20;
			}
			else
			{
				nav.fullscreen.visible = false;
			}
			nav.notify.x = nav.volumeBar.x + nav.volumeBar.width + 10;
			if (autoplay)
			{
				nav.playButton.visible = false;
			}
			else
			{
				nav.pauseButton.visible = false;
			}
			nav.volumn_btn.x = nav.volumeBar.x + 36;
			nav.y = overlay.height - 26;
			nav.playingBar.width = nav.bar.width = W;
			nav.progress_line.x = -30;
			nav.seeker_time.y = nav.progressBar.y - 20;
			nav.seeker_time.x = 0;
			nav.hover_line.y = nav.progress_line.y = nav.progressBar.y;
			nav.setChildIndex(nav.progress_line,nav.numChildren-1);
			nav.setChildIndex(nav.hover_line,nav.numChildren-1);
			nav.hover_line.visible = false;
			nav.hover_line.buttonMode = true;
			nav.progress_line.buttonMode = true;
			nav.progressBar.buttonMode = true;
			nav.playingBar.buttonMode = true;
		}
		private function formatTime(time:Number):String
		{
			if (time > 0)
			{
				var integer:String = String((time / 60) >> 0);
				var decimal:String = String((time % 60) >> 0);
				return ((integer.length < 2) ? "0" + integer : integer) + ":" + ((decimal.length < 2) ? "0" + decimal : decimal);
			}
			else
			{
				return String("00:00");
			}
		}
	}

}	